#changelist-legend {
    margin-bottom: 15px;
}

#changelist-legend div {
    padding: 10px;
    margin: 5px;
}

#changelist-legend .reg-new,
.change-list table .row1.reg-new,
.change-list table .row2.reg-new
{
    background: #ddf;
}
.change-list table .row1.reg-new:hover,
.change-list table .row2.reg-new:hover
{
    background: #eef;
}
#changelist-legend .reg-new.reg-full,
.change-list table .row1.reg-new.reg-full,
.change-list table .row2.reg-new.reg-full
{
    background: repeating-linear-gradient(
        45deg,
        #ddf,
        #ddf 10px,
        #eef 10px,
        #eef 20px
      );
}
#changelist-legend .reg-approved,
.change-list table .row1.reg-approved,
.change-list table .row2.reg-approved
{
    background: #dfd;
}
.change-list table .row1.reg-approved:hover,
.change-list table .row2.reg-approved:hover
{
    background: #efe;
}
#changelist-legend .reg-cancel-request,
.change-list table .row1.reg-cancel-request,
.change-list table .row2.reg-cancel-request
{
    background: #fdd;
}
.change-list table .row1.reg-cancel-request:hover,
.change-list table .row2.reg-cancel-request:hover
{
    background: #fee;
}
