{% load i18n %}
<p>
    {% trans 'You may check the current status of your registrations and payments using following link:' %}<br/>
    <a href="{{ site.url }}{% url 'leprikon:registration_list' %}">
        {{ site.url }}{% url 'leprikon:registration_list' %}
    </a>
</p>

<p>{{ site.name }}</p>
