# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2022-02-15 23:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("leprikon", "0066_subject_type_slug_unique"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="subject",
            name="chat_group_type",
        ),
        migrations.RemoveField(
            model_name="subjecttype",
            name="chat_group_type",
        ),
    ]
