{% load i18n leprikon_tags qr_code %}

<div class="row">
    <label class="col-sm-4">{% trans 'Presence' %}:</label>
    <div class="col-sm-8">
        {% for entry, present in participant.presences %}
        <strong title="{{ entry.date }}" style="color: #{% if present %}0a0{% else %}e00{% endif %}">
            {% if present %}&#x2713;{% else %}&#x2717;{% endif %}
        </strong>
        {% endfor %}
    </div>
</div>
