{% extends 'leprikon/list.html' %}
{% load cms_tags i18n leprikon_tags %}

{% block list %}

<table class="table">
    <tr>
        <th>{% trans 'Date' %}</th>
        <th>{% trans 'Donation Type' %}</th>
        <th>{% trans 'Organization' %}</th>
        <th>{% trans 'Amount' %}</th>
        <th></th>
    </tr>
{% for donation in object_list %}
    <tr>
        <td>{{ donation.accounted.date }}</td>
        <td>{{ donation.transaction_type_label }}</td>
        <td>{{ donation.organization }}</td>
        <td>{{ donation.amount | currency }}</td>
        <td>
            <a class="btn btn-primary btn-xs" href="{{ donation.get_absolute_url }}">
                <span class="glyphicon glyphicon-download" aria-hidden="true"></span>
                {% trans 'download' %}
            </a>
        </td>
    </tr>
{% empty %}
    <tr>
        <td colspan="5">{{ message_empty }}</td>
    </tr>
{% endfor %}
</table>
{% endblock %}
