# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-09-28 06:27
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0034_subject_registration_group'),
    ]

    operations = [
        migrations.AlterField(
            model_name='course',
            name='school_year_division',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='courses', to='leprikon.SchoolYearDivision', verbose_name='school year division'),
        ),
        migrations.AlterField(
            model_name='coursediscount',
            name='registration',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='discounts', to='leprikon.CourseRegistration', verbose_name='registration'),
        ),
        migrations.AlterField(
            model_name='eventdiscount',
            name='registration',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='discounts', to='leprikon.EventRegistration', verbose_name='registration'),
        ),
        migrations.AlterField(
            model_name='subjectpayment',
            name='bankreader_transaction',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bankreader.Transaction', verbose_name='bank account transaction'),
        ),
        migrations.AlterField(
            model_name='subjectpayment',
            name='pays_payment',
            field=models.OneToOneField(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to='django_pays.Payment', verbose_name='online payment'),
        ),
    ]
